# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/sql-face',
                'doc_host': 'https://AndreaMacarulla.github.io',
                'git_url': 'https://github.com/AndreaMacarulla/sql-face',
                'lib_path': 'sql_face'},
  'syms': { 'sql_face.alchemy': { 'sql_face.alchemy.create_cropped_images': ('alchemy.html#create_cropped_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.create_detectors': ('alchemy.html#create_detectors', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.create_embedding_models': ( 'alchemy.html#create_embedding_models',
                                                                                'sql_face/alchemy.py'),
                                  'sql_face.alchemy.create_face_images': ('alchemy.html#create_face_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.create_quality_images': ('alchemy.html#create_quality_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.create_quality_images_new': ( 'alchemy.html#create_quality_images_new',
                                                                                  'sql_face/alchemy.py'),
                                  'sql_face.alchemy.create_quality_models': ('alchemy.html#create_quality_models', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.fill_cropped_image_general': ( 'alchemy.html#fill_cropped_image_general',
                                                                                   'sql_face/alchemy.py'),
                                  'sql_face.alchemy.fill_cropped_image_serfiq': ( 'alchemy.html#fill_cropped_image_serfiq',
                                                                                  'sql_face/alchemy.py'),
                                  'sql_face.alchemy.get_session': ('alchemy.html#get_session', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_age': ('alchemy.html#update_age', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_angles': ('alchemy.html#update_angles', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_cropped_images': ('alchemy.html#update_cropped_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_embeddings_arcface': ( 'alchemy.html#update_embeddings_arcface',
                                                                                  'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_embeddings_deepface': ( 'alchemy.html#update_embeddings_deepface',
                                                                                   'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_embeddings_qmagface': ( 'alchemy.html#update_embeddings_qmagface',
                                                                                   'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_emotion': ('alchemy.html#update_emotion', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_face_images': ('alchemy.html#update_face_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_gender': ('alchemy.html#update_gender', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_images': ('alchemy.html#update_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_pose': ('alchemy.html#update_pose', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_quality_images': ('alchemy.html#update_quality_images', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_race': ('alchemy.html#update_race', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_ser_fiq': ('alchemy.html#update_ser_fiq', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_ser_fiq_old': ('alchemy.html#update_ser_fiq_old', 'sql_face/alchemy.py'),
                                  'sql_face.alchemy.update_tface': ('alchemy.html#update_tface', 'sql_face/alchemy.py')},
            'sql_face.core': {'sql_face.core.foo': ('core.html#foo', 'sql_face/core.py')},
            'sql_face.databases': { 'sql_face.databases.ChokePoint': ('databases.html#chokepoint', 'sql_face/databases.py'),
                                    'sql_face.databases.ChokePoint.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.ChokePoint.create_images': ( 'databases.html#create_images',
                                                                                     'sql_face/databases.py'),
                                    'sql_face.databases.ChokePoint.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                           'sql_face/databases.py'),
                                    'sql_face.databases.ChokePoint.get_groundtruth': ( 'databases.html#get_groundtruth',
                                                                                       'sql_face/databases.py'),
                                    'sql_face.databases.ChokePoint.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                          'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi': ('databases.html#enfsi', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.create_images': ('databases.html#create_images', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.fill_enfsipair': ('databases.html#fill_enfsipair', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.fill_qry_ref': ('databases.html#fill_qry_ref', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                      'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.get_query_reference': ( 'databases.html#get_query_reference',
                                                                                      'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                     'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015': ('databases.html#enfsi2015', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.create_images': ('databases.html#create_images', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.fill_enfsipair2015': ( 'databases.html#fill_enfsipair2015',
                                                                                         'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.fill_query': ('databases.html#fill_query', 'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.fill_reference': ( 'databases.html#fill_reference',
                                                                                     'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                          'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.get_query_reference': ( 'databases.html#get_query_reference',
                                                                                          'sql_face/databases.py'),
                                    'sql_face.databases.Enfsi2015.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                         'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase': ('databases.html#facedatabase', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.__eq__': ('databases.html#__eq__', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.__hash__': ('databases.html#__hash__', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.__str__': ('databases.html#__str__', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.create_images': ( 'databases.html#create_images',
                                                                                       'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                             'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.get_path': ('databases.html#get_path', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                            'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.paths_in_db': ('databases.html#paths_in_db', 'sql_face/databases.py'),
                                    'sql_face.databases.FaceDataBase.paths_not_in_db': ( 'databases.html#paths_not_in_db',
                                                                                         'sql_face/databases.py'),
                                    'sql_face.databases.ForenFace': ('databases.html#forenface', 'sql_face/databases.py'),
                                    'sql_face.databases.ForenFace.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.ForenFace.create_images': ('databases.html#create_images', 'sql_face/databases.py'),
                                    'sql_face.databases.ForenFace.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                          'sql_face/databases.py'),
                                    'sql_face.databases.ForenFace.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                         'sql_face/databases.py'),
                                    'sql_face.databases.ForenFace.set_attributes': ( 'databases.html#set_attributes',
                                                                                     'sql_face/databases.py'),
                                    'sql_face.databases.LFW': ('databases.html#lfw', 'sql_face/databases.py'),
                                    'sql_face.databases.LFW.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.LFW.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                    'sql_face/databases.py'),
                                    'sql_face.databases.LFW.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                   'sql_face/databases.py'),
                                    'sql_face.databases.SCFace': ('databases.html#scface', 'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.SCFace._types_to_folders': ( 'databases.html#_types_to_folders',
                                                                                     'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.create_images': ('databases.html#create_images', 'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                       'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                      'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.set_glasses_beard_gender': ( 'databases.html#set_glasses_beard_gender',
                                                                                            'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.set_yaw_pitch_dist': ( 'databases.html#set_yaw_pitch_dist',
                                                                                      'sql_face/databases.py'),
                                    'sql_face.databases.SCFace.type_from_path': ('databases.html#type_from_path', 'sql_face/databases.py'),
                                    'sql_face.databases.UTKFace': ('databases.html#utkface', 'sql_face/databases.py'),
                                    'sql_face.databases.UTKFace.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.UTKFace.create_images': ('databases.html#create_images', 'sql_face/databases.py'),
                                    'sql_face.databases.UTKFace.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                        'sql_face/databases.py'),
                                    'sql_face.databases.UTKFace.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                       'sql_face/databases.py'),
                                    'sql_face.databases.UTKFace.set_age_gender_race': ( 'databases.html#set_age_gender_race',
                                                                                        'sql_face/databases.py'),
                                    'sql_face.databases.XQLFW': ('databases.html#xqlfw', 'sql_face/databases.py'),
                                    'sql_face.databases.XQLFW.__init__': ('databases.html#__init__', 'sql_face/databases.py'),
                                    'sql_face.databases.XQLFW.get_all_image_paths': ( 'databases.html#get_all_image_paths',
                                                                                      'sql_face/databases.py'),
                                    'sql_face.databases.XQLFW.identity_from_path': ( 'databases.html#identity_from_path',
                                                                                     'sql_face/databases.py'),
                                    'sql_face.databases.get_image_db': ('databases.html#get_image_db', 'sql_face/databases.py')},
            'sql_face.faceangles': { 'sql_face.faceangles.FaceParser': ('faceangles.html#faceparser', 'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__get_body_face_key_landmarks': ( 'faceangles.html#__get_body_face_key_landmarks',
                                                                                                       'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__get_face_bounding_box': ( 'faceangles.html#__get_face_bounding_box',
                                                                                                 'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__get_face_key_landmarks': ( 'faceangles.html#__get_face_key_landmarks',
                                                                                                  'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__get_head_angles': ( 'faceangles.html#__get_head_angles',
                                                                                           'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__init__': ('faceangles.html#__init__', 'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__normalized_to_pixel_coordinates': ( 'faceangles.html#__normalized_to_pixel_coordinates',
                                                                                                           'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.__truncate_to_valid_rect': ( 'faceangles.html#__truncate_to_valid_rect',
                                                                                                  'sql_face/faceangles.py'),
                                     'sql_face.faceangles.FaceParser.detect_faces_lms_ang': ( 'faceangles.html#detect_faces_lms_ang',
                                                                                              'sql_face/faceangles.py'),
                                     'sql_face.faceangles.classify_angle': ('faceangles.html#classify_angle', 'sql_face/faceangles.py'),
                                     'sql_face.faceangles.compute_angles': ('faceangles.html#compute_angles', 'sql_face/faceangles.py'),
                                     'sql_face.faceangles.compute_pose': ('faceangles.html#compute_pose', 'sql_face/faceangles.py')},
            'sql_face.qmagface': { 'sql_face.qmagface.IBasicBlock': ('qmagface.html#ibasicblock', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.IBasicBlock.__init__': ('qmagface.html#__init__', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.IBasicBlock.forward': ('qmagface.html#forward', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.IResNet': ('qmagface.html#iresnet', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.IResNet.__init__': ('qmagface.html#__init__', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.IResNet._make_layer': ('qmagface.html#_make_layer', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.IResNet.forward': ('qmagface.html#forward', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.NetworkBuilder_inf': ('qmagface.html#networkbuilder_inf', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.NetworkBuilder_inf.__init__': ('qmagface.html#__init__', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.NetworkBuilder_inf.forward': ('qmagface.html#forward', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface._iresnet': ('qmagface.html#_iresnet', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.builder_inf': ('qmagface.html#builder_inf', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.clean_dict_inf': ('qmagface.html#clean_dict_inf', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.compute_qmagface_embeddings': ( 'qmagface.html#compute_qmagface_embeddings',
                                                                                      'sql_face/qmagface.py'),
                                   'sql_face.qmagface.conv1x1': ('qmagface.html#conv1x1', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.conv3x3': ('qmagface.html#conv3x3', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.iresnet100': ('qmagface.html#iresnet100', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.iresnet18': ('qmagface.html#iresnet18', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.iresnet34': ('qmagface.html#iresnet34', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.iresnet50': ('qmagface.html#iresnet50', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.load_dict_inf': ('qmagface.html#load_dict_inf', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.load_features': ('qmagface.html#load_features', 'sql_face/qmagface.py'),
                                   'sql_face.qmagface.load_model': ('qmagface.html#load_model', 'sql_face/qmagface.py')},
            'sql_face.serfiq': { 'sql_face.serfiq.MtcnnDetector': ('serfiq.html#mtcnndetector', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.__init__': ('serfiq.html#__init__', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.calibrate_box': ('serfiq.html#calibrate_box', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.convert_to_square': ('serfiq.html#convert_to_square', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.detect_face': ('serfiq.html#detect_face', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.detect_face_limited': ( 'serfiq.html#detect_face_limited',
                                                                                        'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.extract_image_chips': ( 'serfiq.html#extract_image_chips',
                                                                                        'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.find_tfrom_between_shapes': ( 'serfiq.html#find_tfrom_between_shapes',
                                                                                              'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.list2colmatrix': ('serfiq.html#list2colmatrix', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.pad': ('serfiq.html#pad', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.MtcnnDetector.slice_index': ('serfiq.html#slice_index', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.SER_FIQ': ('serfiq.html#ser_fiq', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.SER_FIQ.__init__': ('serfiq.html#__init__', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.SER_FIQ.apply_mtcnn': ('serfiq.html#apply_mtcnn', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.SER_FIQ.get_score': ('serfiq.html#get_score', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.adjust_input': ('serfiq.html#adjust_input', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.detect_first_stage': ('serfiq.html#detect_first_stage', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.detect_first_stage_warpper': ( 'serfiq.html#detect_first_stage_warpper',
                                                                                 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.generate_bbox': ('serfiq.html#generate_bbox', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.get_serfiq_model': ('serfiq.html#get_serfiq_model', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.nms': ('serfiq.html#nms', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.parse_lst_line': ('serfiq.html#parse_lst_line', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.preprocess': ('serfiq.html#preprocess', 'sql_face/serfiq.py'),
                                 'sql_face.serfiq.read_image': ('serfiq.html#read_image', 'sql_face/serfiq.py')},
            'sql_face.sqldb': { 'sql_face.sqldb.SQLDataBase': ('sqldb.html#sqldatabase', 'sql_face/sqldb.py'),
                                'sql_face.sqldb.SQLDataBase.__init__': ('sqldb.html#__init__', 'sql_face/sqldb.py'),
                                'sql_face.sqldb.SQLDataBase.create_tables': ('sqldb.html#create_tables', 'sql_face/sqldb.py'),
                                'sql_face.sqldb.SQLDataBase.update_tables': ('sqldb.html#update_tables', 'sql_face/sqldb.py'),
                                'sql_face.sqldb.get_output_dir': ('sqldb.html#get_output_dir', 'sql_face/sqldb.py')},
            'sql_face.tables': { 'sql_face.tables.Age': ('tables.html#age', 'sql_face/tables.py'),
                                 'sql_face.tables.Age.age2enum': ('tables.html#age2enum', 'sql_face/tables.py'),
                                 'sql_face.tables.CPFrame': ('tables.html#cpframe', 'sql_face/tables.py'),
                                 'sql_face.tables.CPFrame.get_image': ('tables.html#get_image', 'sql_face/tables.py'),
                                 'sql_face.tables.CroppedImage': ('tables.html#croppedimage', 'sql_face/tables.py'),
                                 'sql_face.tables.CroppedImage.get_aligned_image': ('tables.html#get_aligned_image', 'sql_face/tables.py'),
                                 'sql_face.tables.CroppedImage.get_cropped_image': ('tables.html#get_cropped_image', 'sql_face/tables.py'),
                                 'sql_face.tables.Detector': ('tables.html#detector', 'sql_face/tables.py'),
                                 'sql_face.tables.Distance': ('tables.html#distance', 'sql_face/tables.py'),
                                 'sql_face.tables.EmbeddingModel': ('tables.html#embeddingmodel', 'sql_face/tables.py'),
                                 'sql_face.tables.Emotion': ('tables.html#emotion', 'sql_face/tables.py'),
                                 'sql_face.tables.EnfsiImage': ('tables.html#enfsiimage', 'sql_face/tables.py'),
                                 'sql_face.tables.EnfsiMixin': ('tables.html#enfsimixin', 'sql_face/tables.py'),
                                 'sql_face.tables.EnfsiMixin.image_id': ('tables.html#image_id', 'sql_face/tables.py'),
                                 'sql_face.tables.EnfsiPair': ('tables.html#enfsipair', 'sql_face/tables.py'),
                                 'sql_face.tables.EnfsiPair2015': ('tables.html#enfsipair2015', 'sql_face/tables.py'),
                                 'sql_face.tables.EnfsiVideoFrame': ('tables.html#enfsivideoframe', 'sql_face/tables.py'),
                                 'sql_face.tables.FaceImage': ('tables.html#faceimage', 'sql_face/tables.py'),
                                 'sql_face.tables.Gender': ('tables.html#gender', 'sql_face/tables.py'),
                                 'sql_face.tables.Image': ('tables.html#image', 'sql_face/tables.py'),
                                 'sql_face.tables.Image.get_category': ('tables.html#get_category', 'sql_face/tables.py'),
                                 'sql_face.tables.Image.get_fi_category': ('tables.html#get_fi_category', 'sql_face/tables.py'),
                                 'sql_face.tables.Image.get_im_category': ('tables.html#get_im_category', 'sql_face/tables.py'),
                                 'sql_face.tables.Image.get_image': ('tables.html#get_image', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair': ('tables.html#pair', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.__init__': ('tables.html#__init__', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.get_category': ('tables.html#get_category', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.get_n_common_attributes': ( 'tables.html#get_n_common_attributes',
                                                                                   'sql_face/tables.py'),
                                 'sql_face.tables.Pair.is_same': ('tables.html#is_same', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.is_valid': ('tables.html#is_valid', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.make_cropped_pair': ('tables.html#make_cropped_pair', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.make_face_image_pair': ('tables.html#make_face_image_pair', 'sql_face/tables.py'),
                                 'sql_face.tables.Pair.pair_category_str': ('tables.html#pair_category_str', 'sql_face/tables.py'),
                                 'sql_face.tables.Pitch': ('tables.html#pitch', 'sql_face/tables.py'),
                                 'sql_face.tables.QualityGroup': ('tables.html#qualitygroup', 'sql_face/tables.py'),
                                 'sql_face.tables.QualityImage': ('tables.html#qualityimage', 'sql_face/tables.py'),
                                 'sql_face.tables.QualityModel': ('tables.html#qualitymodel', 'sql_face/tables.py'),
                                 'sql_face.tables.Race': ('tables.html#race', 'sql_face/tables.py'),
                                 'sql_face.tables.Roll': ('tables.html#roll', 'sql_face/tables.py'),
                                 'sql_face.tables.SCFaceMixin': ('tables.html#scfacemixin', 'sql_face/tables.py'),
                                 'sql_face.tables.SCFaceMixin.image_id': ('tables.html#image_id', 'sql_face/tables.py'),
                                 'sql_face.tables.SCImage': ('tables.html#scimage', 'sql_face/tables.py'),
                                 'sql_face.tables.VideoFrame': ('tables.html#videoframe', 'sql_face/tables.py'),
                                 'sql_face.tables.VideoMixin': ('tables.html#videomixin', 'sql_face/tables.py'),
                                 'sql_face.tables.VideoMixin.get_image': ('tables.html#get_image', 'sql_face/tables.py'),
                                 'sql_face.tables.VideoMixin.image_id': ('tables.html#image_id', 'sql_face/tables.py'),
                                 'sql_face.tables.Yaw': ('tables.html#yaw', 'sql_face/tables.py')},
            'sql_face.tface': { 'sql_face.tface.Backbone': ('tface.html#backbone', 'sql_face/tface.py'),
                                'sql_face.tface.Backbone.__init__': ('tface.html#__init__', 'sql_face/tface.py'),
                                'sql_face.tface.Backbone._initialize_weights': ('tface.html#_initialize_weights', 'sql_face/tface.py'),
                                'sql_face.tface.Backbone.forward': ('tface.html#forward', 'sql_face/tface.py'),
                                'sql_face.tface.Bottleneck': ('tface.html#bottleneck', 'sql_face/tface.py'),
                                'sql_face.tface.Flatten': ('tface.html#flatten', 'sql_face/tface.py'),
                                'sql_face.tface.Flatten.forward': ('tface.html#forward', 'sql_face/tface.py'),
                                'sql_face.tface.R50': ('tface.html#r50', 'sql_face/tface.py'),
                                'sql_face.tface.bottleneck_IR': ('tface.html#bottleneck_ir', 'sql_face/tface.py'),
                                'sql_face.tface.bottleneck_IR.__init__': ('tface.html#__init__', 'sql_face/tface.py'),
                                'sql_face.tface.bottleneck_IR.forward': ('tface.html#forward', 'sql_face/tface.py'),
                                'sql_face.tface.compute_tf_quality': ('tface.html#compute_tf_quality', 'sql_face/tface.py'),
                                'sql_face.tface.get_block': ('tface.html#get_block', 'sql_face/tface.py'),
                                'sql_face.tface.get_blocks': ('tface.html#get_blocks', 'sql_face/tface.py'),
                                'sql_face.tface.get_network': ('tface.html#get_network', 'sql_face/tface.py'),
                                'sql_face.tface.l2_norm': ('tface.html#l2_norm', 'sql_face/tface.py'),
                                'sql_face.tface.network': ('tface.html#network', 'sql_face/tface.py'),
                                'sql_face.tface.preprocess_tf_img': ('tface.html#preprocess_tf_img', 'sql_face/tface.py')}}}


# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/09_faceangles.ipynb.

# %% auto 0
__all__ = ['FaceParser', 'compute_angles', 'compute_pose']

# %% ../nbs/09_faceangles.ipynb 4
import math
import numpy as np
import cv2
import mediapipe as mp
from mediapipe.framework.formats import landmark_pb2
from skimage import transform as trans
from typing import Tuple, Union, List
from scipy.spatial.transform import Rotation as R

from sql_face.tables import  Yaw, Pitch, Roll

# %% ../nbs/09_faceangles.ipynb 5
class FaceParser():
    def __init__(self, max_num_faces=1, min_detection_confidence=0.5,
                 min_tracking_confidence=0.5, static_image_mode=True):
        self.__mp_face_mesh = mp.solutions.face_mesh
        self.__face_mesh = self.__mp_face_mesh.FaceMesh(
            max_num_faces=max_num_faces,
            min_detection_confidence=min_detection_confidence,
            min_tracking_confidence=min_tracking_confidence,
            static_image_mode=static_image_mode)
        self.__presence_thr = 0.5
        self.__visibility_thr = 0.5

        self.__mp_pose = mp.solutions.pose
        self.__body_pose = self.__mp_pose.Pose(
            static_image_mode=static_image_mode,
            model_complexity=2,
            enable_segmentation=False,
            min_detection_confidence=min_detection_confidence,
            min_tracking_confidence=min_tracking_confidence)

    @staticmethod
    def __normalized_to_pixel_coordinates(
            normalized_x: float, normalized_y: float, image_width: int,
            image_height: int) -> Union[None, Tuple[int, int]]:
        """Converts normalized value pair to pixel coordinates."""

        # Checks if the float value is between 0 and 1.
        def is_valid_normalized_value(value: float) -> bool:
            return (value > 0 or math.isclose(0, value)) and \
                (value < 1 or math.isclose(1, value))

        if not (is_valid_normalized_value(normalized_x) and
                is_valid_normalized_value(normalized_y)):
            # TODO: Draw coordinates even if it's outside of the image bounds.
            return None
        x_px = min(math.floor(normalized_x * image_width), image_width - 1)
        y_px = min(math.floor(normalized_y * image_height), image_height - 1)
        return x_px, y_px

    def __get_face_key_landmarks(self,
                                 face_landmarks: landmark_pb2.NormalizedLandmarkList,
                                 image_cols: int, image_rows: int):
        key_landmarks = {}
        norm_key_landmarks = {}
        for idx, landmark in enumerate(face_landmarks.landmark):
            if idx not in (4, 33, 61, 129, 133, 168, 263, 308, 358, 362):
                continue
            if ((landmark.HasField('visibility') and
                landmark.visibility < self.__visibility_thr) or
                (landmark.HasField('presence') and
                    landmark.presence < self.__presence_thr)):
                continue
            landmark_px = self.__normalized_to_pixel_coordinates(
                normalized_x=landmark.x,
                normalized_y=landmark.y,
                image_width=image_cols,
                image_height=image_rows)
            if landmark_px:
                key_landmarks[idx] = (landmark_px, landmark.z)
                norm_key_landmarks[idx] = (landmark.x, landmark.y, landmark.z)

        face_key_landmarks = {}
        norm_face_key_landmarks = {}
        # Face landmarks:
        # 0: Left eye outer corner (33)
        # 1: Left eye inner corner (133)
        # 2: Right eye inner corner (362)
        # 3: Right eye outer corner (263)
        # 4: Nose tip (4)
        # 5: Mouth left corner (61)
        # 6: Mouth right corner (308)
        if 33 in key_landmarks:
            face_key_landmarks[0] = key_landmarks[33]
            norm_face_key_landmarks[0] = norm_key_landmarks[33]
        if 133 in key_landmarks:
            face_key_landmarks[1] = key_landmarks[133]
            norm_face_key_landmarks[1] = norm_key_landmarks[133]
        if 362 in key_landmarks:
            face_key_landmarks[2] = key_landmarks[362]
            norm_face_key_landmarks[2] = norm_key_landmarks[362]
        if 263 in key_landmarks:
            face_key_landmarks[3] = key_landmarks[263]
            norm_face_key_landmarks[3] = norm_key_landmarks[263]
        if 4 in key_landmarks:
            face_key_landmarks[4] = key_landmarks[4]
            norm_face_key_landmarks[4] = norm_key_landmarks[4]
        if 61 in key_landmarks:
            face_key_landmarks[5] = key_landmarks[61]
            norm_face_key_landmarks[5] = norm_key_landmarks[61]
        if 308 in key_landmarks:
            face_key_landmarks[6] = key_landmarks[308]
            norm_face_key_landmarks[6] = norm_key_landmarks[308]

        return face_key_landmarks, norm_face_key_landmarks

    def __get_body_face_key_landmarks(self, image):
        face_key_landmarks = {}
        norm_face_key_landmarks = {}
        results = self.__body_pose.process(image)

        if not results.pose_landmarks:
            return face_key_landmarks, norm_face_key_landmarks

        # Face landmarks:
        # 0: Left eye outer corner
        # 1: Left eye inner corner
        # 2: Right eye inner corner
        # 3: Right eye outer corner
        # 4: Nose tip
        # 5: Mouth left corner
        # 6: Mouth right corner
        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.RIGHT_EYE_OUTER]
        norm_face_key_landmarks[0] = (coords.x, coords.y, coords.z)
        face_key_landmarks[0] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.RIGHT_EYE_INNER]
        norm_face_key_landmarks[1] = (coords.x, coords.y, coords.z)
        face_key_landmarks[1] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.LEFT_EYE_INNER]
        norm_face_key_landmarks[2] = (coords.x, coords.y, coords.z)
        face_key_landmarks[2] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.LEFT_EYE_OUTER]
        norm_face_key_landmarks[3] = (coords.x, coords.y, coords.z)
        face_key_landmarks[3] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.NOSE]
        norm_face_key_landmarks[4] = (coords.x, coords.y, coords.z)
        face_key_landmarks[4] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.MOUTH_RIGHT]
        norm_face_key_landmarks[5] = (coords.x, coords.y, coords.z)
        face_key_landmarks[5] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        coords = results.pose_landmarks.landmark[
            self.__mp_pose.PoseLandmark.MOUTH_LEFT]
        norm_face_key_landmarks[6] = (coords.x, coords.y, coords.z)
        face_key_landmarks[6] = \
            ((coords.x * image.shape[1], coords.y * image.shape[0]),
             coords.visibility)

        return face_key_landmarks, norm_face_key_landmarks

    @staticmethod
    def __truncate_to_valid_rect(rect, img_w, img_h):
        tl_x = int(max(0, min(img_w - 1, rect[0])))
        tl_y = int(max(0, min(img_h - 1, rect[1])))
        br_x = int(max(0, min(img_w, rect[0] + rect[2])))
        br_y = int(max(0, min(img_h, rect[1] + rect[3])))
        width = int(max(0, br_x - tl_x))
        height = int(max(0, br_y - tl_y))
        return [tl_x, tl_y, width, height]

    @staticmethod
    def __get_face_bounding_box(key_landmarks, image_cols, image_rows):
        min_x = image_cols
        min_y = image_rows
        max_x = 0
        max_y = 0
        for key_landmark in key_landmarks.values():
            if key_landmark[0][0] <= min_x:
                min_x = key_landmark[0][0]
            if key_landmark[0][1] <= min_y:
                min_y = key_landmark[0][1]
            if key_landmark[0][0] >= max_x:
                max_x = key_landmark[0][0]
            if key_landmark[0][1] >= max_y:
                max_y = key_landmark[0][1]
        mean_x = (min_x + max_x) / 2
        mean_y = (min_y + max_y) / 2
        width = 2.65 * ((max_x - min_x) + (max_y - min_y)) / 2
        height = width

        return [int(mean_x - 0.52 * width), int(mean_y - 0.54 * height),
                int(width), int(height)]

    def __get_head_angles(self, norm_key_landmarks):
        # Camera reference: +X left, +Y up, and +Z backwards (right handed)
        lids = [0, 1, 2]
        rids = [3, 2, 1]

        lid = None
        for id in lids:
            if id in norm_key_landmarks:
                lid = id
                break
        for id in rids:
            if id in norm_key_landmarks:
                rid = id
                break

        # If landmark ids are not in list, return default values
        lands_missed = len(norm_key_landmarks) < 7
        if lid is None or lands_missed:
            rot = R.identity()
            return rot.as_euler('xyz', degrees=True)

        if lid != rid and lid in norm_key_landmarks and \
                rid in norm_key_landmarks:
            x = np.array(np.array(norm_key_landmarks[rid]) -
                         np.array(norm_key_landmarks[lid]))
            x /= np.linalg.norm(x)
        else:
            x = np.array([1.0, 0, 0])
        if 1 in norm_key_landmarks and 2 in norm_key_landmarks and \
                5 in norm_key_landmarks and 6 in norm_key_landmarks:
            y = np.array(0.5 * (np.array(norm_key_landmarks[5]) +
                                np.array(norm_key_landmarks[6])) -
                         0.5 * (np.array(norm_key_landmarks[1]) +
                                np.array(norm_key_landmarks[2])))
            y /= np.linalg.norm(y)
        else:
            y = np.array([0, -1.0, 0])
        z = np.cross(x, y)
        z /= np.linalg.norm(z)

        rot = R.from_matrix([[x[0], y[0], z[0]],
                             [x[1], y[1], z[1]],
                             [x[2], y[2], z[2]]])

        return rot.as_euler('xyz', degrees=True)

    def detect_faces_lms_ang(self, image):
        """
        Detects faces, facial landmarks and head angles on the image.
        """
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        image.flags.writeable = False
        results = self.__face_mesh.process(image)
        image_rows, image_cols, _ = image.shape
        faces = []
        landmarks = []
        angles = []
        if results.multi_face_landmarks:
            for face_landmarks in results.multi_face_landmarks:
                key_landmarks, norm_key_landmarks = \
                    self.__get_face_key_landmarks(
                        face_landmarks, image_cols, image_rows)
                # key_landmarks, norm_key_landmarks = \
                #     self.__get_body_face_key_landmarks(image)

                bounding_box = self.__get_face_bounding_box(
                    key_landmarks, image_cols, image_rows)
                bounding_box = self.__truncate_to_valid_rect(
                    bounding_box, image_cols, image_rows)

                head_angles = self.__get_head_angles(norm_key_landmarks)

                landmarks.append(key_landmarks)
                faces.append(bounding_box)
                angles.append(head_angles.tolist())

        return faces, landmarks, angles

# %% ../nbs/09_faceangles.ipynb 7
thresholds = {
'pitch': [-16.47307851, -7.950750253, 0.710368788, 6.941161856, np.inf],
'yaw': [10.84743824, 20.52868443, np.inf],
'roll': [6.373882696, 13.32788435, np.inf]
}

# Face parser instance
face_parser = FaceParser()

# %% ../nbs/09_faceangles.ipynb 8
def classify_angle(angle, enum_class, thresholds):
    if angle:
        return next(enum_value for enum_value, threshold in zip(enum_class, thresholds) if angle < threshold)
    return None

# %% ../nbs/09_faceangles.ipynb 9
def compute_angles(img:np.array)->List[float]:

    faces, landmarks, angles_all = face_parser.detect_faces_lms_ang(img)

    if angles_all:
        angles = angles_all[0] #only first face
        return angles
        
    return None, None, None

# %% ../nbs/09_faceangles.ipynb 10
def compute_pose(angles)->[Pitch, Yaw, Roll]:

    #faces, landmarks, angles_all = face_parser.detect_faces_lms_ang(img)
    pitch = None
    yaw = None
    roll = None

    if angles:
        ang_pitch, ang_yaw, ang_roll = angles
        pitch = classify_angle(ang_pitch, Pitch, thresholds['pitch'])
            
        if ang_yaw:
            ang_yaw = abs(ang_yaw)
            yaw = classify_angle(ang_yaw, Yaw, thresholds['yaw'])

        if ang_roll:
            ang_roll = abs(ang_roll)
            roll = classify_angle(ang_roll, Roll, thresholds['roll'])          
        
        

    return pitch, yaw, roll
        
    
